"use strict";
/*!
 * Copyright (c) 2021-present, Okta, Inc. and/or its affiliates. All rights reserved.
 * The Okta software accompanied by this notice is provided pursuant to the Apache License, Version 2.0 (the "License.")
 *
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations under the License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const worker_threads_1 = require("worker_threads");
const postcss_1 = __importDefault(require("postcss"));
const postcss_load_config_1 = __importDefault(require("postcss-load-config"));
const crypto_1 = require("crypto");
const fs_1 = require("fs");
const formatWarnings_1 = __importDefault(require("./formatWarnings"));
if (!worker_threads_1.parentPort) {
    throw new Error("Cannot execute this module in the main thread");
}
worker_threads_1.parentPort.addListener("message", async (message) => {
    let response;
    const { signal, port, filePath } = message;
    try {
        const source = (0, fs_1.readFileSync)(filePath, "utf8");
        let tokens;
        const CONTEXT_KEY = "odyssey";
        const context = {
            [CONTEXT_KEY]: {
                modules: {
                    getJSON(_, _tokens) {
                        tokens = _tokens;
                    },
                },
            },
        };
        const { plugins } = await (0, postcss_load_config_1.default)(context);
        const runner = (0, postcss_1.default)(plugins);
        const result = await runner.process(source, { from: filePath });
        if (!tokens) {
            throw new Error("Cannot fetch style tokens");
        }
        const styles = result.css;
        const digest = (0, crypto_1.createHash)("md5").update(styles).digest("hex").substr(0, 6);
        response = {
            digest,
            styles,
            tokens,
            warning: (0, formatWarnings_1.default)(result.warnings()),
        };
    }
    catch (error) {
        response = { error };
    }
    finally {
        port.postMessage(response);
        port.close();
        Atomics.store(signal, 0, 1);
        Atomics.notify(signal, 0);
    }
});
